#include <s_globals.h>
#include <targets.h>
#include <cleanup.h>

p_target tgs=NULL;
mlock_t tgslock;

void target_cleanup(int code, void *priv)
{
	p_target foo,bar;
	
	print_raw("Removing targets" CRLF);
	
	if ( tgs==NULL ) return;

	CRITICAL_START(tgslock);
	for (foo=tgs; foo; )
	{
		bar=foo;
		foo=(p_target)foo->l.next;
		free(bar);
		bar=NULL;
	}
	CRITICAL_STOP(tgslock);
	
	return;
}

int target_init(void)
{
	MUTEX_CREATE(tgslock, "Targets");
	cleanup_add_handler(cleanup_lock, (void *)&tgslock);

	cleanup_add_handler(target_cleanup, NULL);
	return 1;
}

p_target target_get(char *name)
{
	p_target p;

	for (p=tgs; p; p=(p_target)p->l.next)
	{
		if ( strcasecmp(name, p->name) == 0 ) {
			return p;
		}
	}

	return NULL;
}

/* Plugins call this to add a new target */
int target_add(char *name, proc_target tg,
	proc_target_validate v,
	proc_target_cleanup c)
{
	p_target newt=NULL;
	
	if ( !name || !tg || !v || !c ) return 0;

	if ( !(newt=malloc(sizeof(target_t))) ) return 0;
	
	newt->name=name;
	newt->target=tg;
	newt->validate=v;
	newt->cleanup=c;
	
	CRITICAL_START(tgslock);
	tgs=list_append(newt, tgs);
	globals.num_targets++;
	CRITICAL_STOP(tgslock);

	return 1;
}
