#include <alert.h>
#include <s_util.h>

proc_print_out		print_out;
proc_print_raw		print_raw;
proc_target_add		add_target;

char *format_ip(uint ip)
{
	char *tmp;
	struct in_addr addr;
	addr.s_addr=ip;
	
	if (!(tmp=inet_ntoa(addr))) return NULL;

	return strdup(tmp);
}

target_action_e alert_target(p_packet p, void *priv, p_rule r)
{
	int i,x;
	char tcpflags[]="FSRPAU12";
	char disp[9];
	
	print_out("%s - ", priv);

	print_out("%s(%s)", p->capdev->capture->name, p->capdev->interface);
	
	for (i=0; p->layer[i].id; i++) {
		print_out("/%s", p->layer[i].name);
	}print_out(" : PORTS %i -> %i" CRLF, p->sport, p->dport);

	if ( p->layer[LAYER_NET].id == 0x0800 ) {
		char *src, *dst;

		src=format_ip(p->layer[LAYER_NET].h.ip->saddr);
		dst=format_ip(p->layer[LAYER_NET].h.ip->daddr);

		print_out(" IP4: SRC=%s DST=%s TTL=%u IP_ID=%u PROTO=%u" CRLF,
			src, dst,
			p->layer[LAYER_NET].h.ip->ttl,
			ntohs(p->layer[LAYER_NET].h.ip->id),
			p->layer[LAYER_NET].h.ip->protocol);

		if ( src ) free(src);
		if ( dst ) free(dst);
	}
	
	if ( p->layer[LAYER_TRANS].id == IPPROTO_TCP ) {
		print_out(" TCP: SEQ=%u ACK=%u WIN=%u URGP=%u [",
			ntohl(p->layer[LAYER_TRANS].h.tcp->seq),
			ntohl(p->layer[LAYER_TRANS].h.tcp->ack),
			ntohs(p->layer[LAYER_TRANS].h.tcp->win),
			ntohs(p->layer[LAYER_TRANS].h.tcp->urp));

		for(i=0,x=1; i<8; i++,x*=2) {
			if (p->layer[LAYER_TRANS].h.tcp->flags.flags & x)  {
				disp[i]=tcpflags[i];
			}else{
				disp[i]='*';
			}
		}disp[8]='\0';
		
		print_out("%s]" CRLF, (char *)&disp);
	}
	
	print_out(CRLF);
	return TARGET_DONE;
}

void *alert_validate(char *token)
{
	char *tmp;
	
	if ( !token ) return NULL;

	if ( !(tmp=strdup((token))) ) {
		return NULL;
	}

	return tmp;
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);

	print_out=in->print_out;
	print_raw=in->print_raw;
	add_target=in->add_target;
	
	out->name="target/alert";
	out->desc="Prints successful matches to screen";
	out->ver_major=1;
	out->ver_minor=0;
	
	add_target("ALERT",
		alert_target,
		alert_validate,
		in->templates.t_cleanup_ptr);
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
